/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.energy;

import de.tud.kom.p2psim.api.energy.Battery;
import de.tudarmstadt.maki.simonstrator.api.Monitor;
import de.tudarmstadt.maki.simonstrator.api.Monitor.Level;
import de.tudarmstadt.maki.simonstrator.api.util.XMLConfigurableConstructor;

/**
 * This class represents a more complex battery.
 * TODO: Implement non-idealities like Peukert's Law and Recovery Effect
 * @author Lukas Wehrstein
 * @version 1.0, 02.03.2022
 */
public class ExtendedBattery extends SimpleBattery implements Battery{
	
	// initial voltage in mV
	protected final double initialVoltage;
	
	// default Voltage of a battery - e.g. AA: 1500mV, LiPo 3700mV 
	protected final double defaultVoltage;
	
	// the current voltage on the battery in mV
	protected double voltage;
	
	// the current current that is charged in mA
	protected double current;
	
	// the batter charge state 
	protected double percentage;
	
	/**
	 * Create a new battery with the given initial voltage in mV and total capacity in
	 * Joule (<b>not</b> uJ).
	 * 
	 * @param capacity
	 * @param initalVoltage
	 */
	@XMLConfigurableConstructor({ "capacity", "initialEnergy", "initialVoltage" })
	public ExtendedBattery(double capacity, double initialEnergy, double initialVoltage) {
		super(capacity, initialEnergy);
		this.initialVoltage = initialVoltage;
		this.defaultVoltage = initialVoltage;
		Monitor.log(ExtendedBattery.class, Level.DEBUG, "Extended Battery created with %s V.", initialVoltage);
	}
	
	/**
	 * Create a new extended Battery with the extended parameters
	 * @param cap_mAh				Nominal Battery Capacity in mAh
	 * @param defaultVoltage		Nominal Battery Voltage in mV, eg. 1500 (AA) or 3700 (LiPo)
	 * @param initialPercentage		initial Battery state in decimal, eg. 100% = 1; 50% = 0.5
	 * @param initialVoltage		initial Battery Voltage in mV
	 */
	@XMLConfigurableConstructor({ "cap_mAh", "defaultVoltage", "initialPercentage", "initialVoltage" })
	public ExtendedBattery(double cap_mAh, double defaultVoltage, double initialPercentage, double initialVoltage) {		
		super(cap_mAh * defaultVoltage * 0.0036, cap_mAh * defaultVoltage * 0.0036 * initialPercentage);
		this.initialVoltage = initialVoltage;
		this.voltage = initialVoltage;
		this.defaultVoltage = defaultVoltage;
		this.percentage = initialPercentage;
		
		this.current = 0;
		Monitor.log(ExtendedBattery.class, Level.INFO, "Extended Battery created with [%smAh, %smV, %s, %smV].", cap_mAh, defaultVoltage, initialPercentage, initialVoltage);
	}
	
	public double getVoltage() {
		return voltage;
	}
	
	@Override
	public void consumeEnergy(double energy) {
		Monitor.log(getClass(), Level.DEBUG, "Extended Battery Energy: %suJ", currentEnergy);
		Monitor.log(ExtendedBattery.class, Level.DEBUG, "Extended Battery was charged with %s uJ", energy);
		if ( energy >= currentEnergy) {
			currentEnergy = 0;
			Monitor.log(ExtendedBattery.class, Level.DEBUG, "Extended Battery is fully discharged");
		} else {
			currentEnergy = currentEnergy - energy;
			Monitor.log(getClass(), Level.DEBUG, "Extended Battery Energy: %suJ", currentEnergy);
		}
	}
	
	@Override
	public ExtendedBattery clone() {
		return new ExtendedBattery(capacity / 1000000, initialEnergy / 1000000, initialVoltage);
	}
	
	@Override
	public ExtendedBattery copy(double initialEnergy) {
		return new ExtendedBattery(capacity / 1000000, initialEnergy / 1000000, initialVoltage);
	}

}
