/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.energy.components;

import de.tud.kom.p2psim.api.energy.ComponentType;
import de.tud.kom.p2psim.api.energy.EnergyCommunicationComponent;
import de.tud.kom.p2psim.api.energy.EnergyEventListener;
import de.tud.kom.p2psim.api.energy.EnergyModel;
import de.tud.kom.p2psim.api.energy.EnergyState;
import de.tud.kom.p2psim.api.linklayer.mac.PhyType;
import de.tud.kom.p2psim.impl.energy.DefaultEnergyState;
import de.tud.kom.p2psim.impl.simengine.Simulator;
import de.tudarmstadt.maki.simonstrator.api.Message;
import de.tudarmstadt.maki.simonstrator.api.Monitor;
import de.tudarmstadt.maki.simonstrator.api.Time;
import de.tudarmstadt.maki.simonstrator.api.Monitor.Level;

/**
 * Communication Component for LoRa Transmissions. States IDLE, SEND, RECEIVE and OFF are configurable via editStates().
 * STate changes can be done with the doStateChange() method.
 * Energy is consumed automatically by using doStateChange(), send() and receive().
 * @author Lukas Wehrstein
 * @version 1.0, 08.03.2022
 */
public class LoRaCommunicationEnergyComponent implements EnergyCommunicationComponent{

	EnergyEventListener energyEventListener;
	
	private EnergyState IDLE, SEND, RECEIVE, OFF;
	private EnergyState currentState;
	private long lastStateChange;

	/**
	 * Constructor without energy state default values - is set to 0. Can be changed with editStates()
	 */
	public LoRaCommunicationEnergyComponent() {
		
		// set default power consumption to 0
		this(0, 0, 0, 0);
	}
	
	/**
	 * Constructor with a list of state energy values. Have to be 4
	 * @param stateEnergyValues
	 */
	public LoRaCommunicationEnergyComponent(double[] stateEnergyValues) {
		this(stateEnergyValues[0], stateEnergyValues[1], stateEnergyValues[2], stateEnergyValues[3]);
		
			
	}
	
	/**
	 * Constructor with default energy state consumption values
	 * @param eIdle
	 * @param eSend
	 * @param eReceive
	 * @param eOff
	 */
	public LoRaCommunicationEnergyComponent(double eIdle, double eSend, double eReceive, double eOff) {
		this.lastStateChange = Simulator.getCurrentTime();
		editStates(eIdle, eSend, eReceive, eOff);
		this.currentState = IDLE;
		
	}
	
	/**
	 * Sets the 
	 * @param eIdle
	 * @param eSend
	 * @param eReceive
	 * @param eOff
	 */
	public void editStates(double eIdle, double eSend, double eReceive, double eOff) {
		if(currentState != null){
			turnOff();
		}
		
		this.IDLE = new DefaultEnergyState("IDLE", eIdle);
		this.SEND = new DefaultEnergyState("SEND", eSend);
		this.RECEIVE = new DefaultEnergyState("RECEIVE", eReceive);
		this.OFF = new DefaultEnergyState("OFF", eOff);
		
		doStateChange(this.IDLE);
	}
	
	/**
	 * Performs State change in new Energy State
	 * @param newState
	 */
	public void doStateChange(EnergyState newState) {
		long timeSpendInState = Simulator.getCurrentTime() - lastStateChange;
		if(timeSpendInState > 0) { // necessary, null pointer on first usage if not used
			// consume energy
			energyEventListener.componentConsumedEnergy(this, calculateEnergyConsumation(currentState, timeSpendInState));
		}
		
		lastStateChange = Simulator.getCurrentTime();
		currentState = newState;
	}

	
	@Override
	public ComponentType getType() {
		return ComponentType.COMMUNICATION;
	}

	@Override
	public boolean turnOff() {
		if(!currentState.equals(OFF)) {
			doStateChange(OFF);
		}
		return true;
	}

	@Override
	public boolean turnOn() {
		if(energyEventListener.componentCanBeActivated(this)) {
			if(!currentState.equals(IDLE)) {
				doStateChange(IDLE);
			}
			return true;
		}else {
			return false;
		}
	}

	@Override
	public boolean isOn() {
		if(currentState.equals(OFF)) {
			return false;
		}else {
			return true;
		}
	}

	@Override
	public void setEnergyEventListener(EnergyEventListener listener) {
		this.energyEventListener = listener;
		
	}

	@Override
	public void eventOccurred(Object content, int type) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public PhyType getPhyType() {
		return PhyType.LORA;
	}

	@Override
	public void send(long duration, Message msg, boolean isBroadcast) {
		// TODO LW configure the right energy consumption
		if(isOn()) {
			Monitor.log(
					getClass(),
					Level.DEBUG,
					Time.getFormattedTime()
					+ " "
					+ ((EnergyModel) energyEventListener).getHost().getHostId()
					+ " consumed "
					+ Math.round(SEND.getEnergyConsumption() * (duration/ (double) Simulator.SECOND_UNIT))
					+ " uJ in State " + SEND.getName() + " after spending "
					+ (duration / (double) Simulator.SECOND_UNIT) + " sec there.");
			// consume energy
			energyEventListener.componentConsumedEnergy(this, calculateEnergyConsumation(SEND, duration));
		}
		
		
	}

	@Override
	public void receive(long duration, Message msg, boolean isBroadcast,
			boolean isIntendedReceiver) {
		if(isOn()) {
			Monitor.log(
					getClass(),
					Level.DEBUG,
					Time.getFormattedTime()
							+ " "
							+ ((EnergyModel) energyEventListener).getHost().getHostId()
							+ " consumed "
							+ (RECEIVE.getEnergyConsumption() * (duration / (double) Simulator.SECOND_UNIT))
							+ " uJ in State " + RECEIVE.getName()
							+ " after spending "
							+ (duration / (double) Simulator.SECOND_UNIT)
							+ " sec there.");
			
			// consume energy
			energyEventListener.componentConsumedEnergy(this, calculateEnergyConsumation(RECEIVE, duration));
		}
		
	}

	@Override
	public void doFakeStateChange() {
		doStateChange(currentState);
	}

}
