/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.energy.components;

import java.util.List;

import de.tud.kom.p2psim.api.energy.ComponentType;
import de.tud.kom.p2psim.api.energy.EnergyComponent;
import de.tud.kom.p2psim.api.energy.EnergyState;
import de.tud.kom.p2psim.impl.energy.DefaultEnergyState;
import de.tud.kom.p2psim.api.energy.EnergyEventListener;
import de.tudarmstadt.maki.simonstrator.api.Event;
import de.tudarmstadt.maki.simonstrator.api.Monitor;
import de.tudarmstadt.maki.simonstrator.api.Time;
import de.tudarmstadt.maki.simonstrator.api.Monitor.Level;

/**
 * Multistate Energy Components enable components to support several energy states with different power consumptions.
 * @author Lukas Wehrstein
 * @version 1.0, 04.03.2022
 */
public class MultiStateEnergyComponent implements EnergyComponent{

	private List<EnergyState> states;
	private EnergyState currentState;
	private boolean on;
	private EnergyEventListener energyEventListener;
	private ComponentType type;
	private long lastConsumption;
	
	/**
	 * Creates a multistateenergyComponent
	 * @param states A list of all states. Not final - states can be dynamically added
	 * @param type	 The Component Type of the Energy Component
	 */
	public MultiStateEnergyComponent(List<EnergyState> states, ComponentType type) {
		this.states = states;
		this.on = true;
		this.type = type;
		
		// take first state as the current state, create a state with consumption 0, if no states are defined.
//		if(states.size()  > 0) {
//			this.currentState = states.get(0);
//		}else {
//			this.currentState = new DefaultEnergyState("default", 0);
//		}
		this.currentState = new DefaultEnergyState("init", 0);
		this.lastConsumption = Time.getCurrentTime();
		Event.scheduleWithDelay(Time.SECOND, this, this, 0);
	}
	
	@Override
	public void eventOccurred(Object content, int id) { // id = -1 means this was an stateChange triggered consumption
		// Just for debugging purpose
		//		if (energyEventListener instanceof AbstractEnergyModel) {
		//			AbstractEnergyModel em = (AbstractEnergyModel) energyEventListener;
		//			Monitor.log(getClass(), Level.DEBUG, "Host %s: %s Event Occured - energy consumption", em.getHost().getId(), Time.getCurrentTime());
		//		}
		
		if(isOn()) {
			// calculate elapsed time since last energy consumption
			long interval = Time.getCurrentTime() - this.lastConsumption;
			double intervalS = ((double) interval) / Time.SECOND;
			
			// calculate the "real" energy consumption
			double consumedEnergy = currentState.getEnergyConsumption() * intervalS;
			
			
			// action to perform, if event occurred
			Monitor.log(getClass(), Level.INFO, "%s consumed %s uJ in %s s.", this.type, consumedEnergy, intervalS);
			// consume the energy
			energyEventListener.componentConsumedEnergy(this, consumedEnergy);
			// set the last consumption time to now
			this.lastConsumption = Time.getCurrentTime();
			
			if (energyEventListener.componentCanBeActivated(this)) {
				if(id == -1) {
					// event occured due to state change - this means do not schedule another event
					Monitor.log(getClass(), Level.DEBUG, "This was an state changed based energy consumption.");
				}else {
					// schedule next static query interval
					Event.scheduleWithDelay(Time.SECOND, this, null, 0);
				}
				
			} else {
				this.turnOff();
			}
			
		}
		
	}
	
	/**
	 * A list of all defined states
	 * @return
	 */
	public List<EnergyState> getStates(){
		return this.states;
	}
	
	/**
	 * adds any number of new energyStates
	 * @param energyStates
	 */
	public void addStates(EnergyState... energyStates) {
		for (EnergyState es : energyStates) {
			this.states.add(es);
		}
	}
	
	/**
	 * Goes to another state, adds the state to the states list, if not already known
	 * @param nextState state to go into
	 * @return true, if state was already in states list, false, if this was a new state
	 */
	public boolean changeState(EnergyState nextState) {
		// search for this state in the states list and make this active.
		Monitor.log(getClass(), Level.DEBUG, "change state into %s", nextState.getName());
		
		boolean wasNewState = true;
		for(EnergyState es : states) {
			if(es.equals(nextState)) {
				currentState = es;
				wasNewState = false;
			}
		}
		
		if(wasNewState) {
			// if not available in list, add this state to list and make it active
			states.add(nextState);
			currentState = nextState;
		}
		
		// consume energy
		Event.scheduleImmediately(this, null, -1); // -1 means, that this is a stateChangePush
		
		return wasNewState;
	}
	
	/**
	 * Returns the first sleep state (state with "sleep" in name) of the states list. Null - if no sleep state is defined
	 * @return energystate with sleep in its name
	 */
	public EnergyState getSleepState() {
		for(EnergyState es : states) {
			if(es.getName().toUpperCase().contains("SLEEP")) {
				return es;
			}
		}
		return null;
	}
	
	/**
	 * Returns the first state with the string active in its name
	 * @return the active state
	 */
	public EnergyState getActiveState() {
		for(EnergyState es : states) {
			if(es.getName().toUpperCase().contains("ACTIVE")) {
				return es;
			}
		}
		return null;
	}

	@Override
	public ComponentType getType() {
		return this.type;
	}

	@Override
	public boolean turnOff() {
		this.on = false;
		return true;
	}

	@Override
	public boolean turnOn() {
		this.on = true;
		return true;
	}

	@Override
	public boolean isOn() {
		return on;
	}

	@Override
	public void setEnergyEventListener(EnergyEventListener listener) {
		this.energyEventListener = listener;
		
	}

}
