/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.energy.components;

import de.tud.kom.p2psim.api.energy.ComponentType;
import de.tud.kom.p2psim.api.energy.EnergyComponent;
import de.tud.kom.p2psim.api.energy.EnergyEventListener;
import de.tud.kom.p2psim.impl.energy.DefaultEnergyState;
import de.tudarmstadt.maki.simonstrator.api.Event;
import de.tudarmstadt.maki.simonstrator.api.Host;
import de.tudarmstadt.maki.simonstrator.api.Monitor;
import de.tudarmstadt.maki.simonstrator.api.Time;
import de.tudarmstadt.maki.simonstrator.api.Monitor.Level;
import de.tudarmstadt.maki.simonstrator.api.util.XMLConfigurableConstructor;

/**
 * This class provides an energy Component with only one state. Ok there are two
 * states (offline), but it gives only one state, which is running all the time.<br>
 * 
 * The component creates an energy component, which consume every second the
 * given power. It is starting with consuming at the creation of this component!
 * 
 * The component can be stopped and startet with the turnOff and turnOn methods
 * of this class.
 * 
 * @author Christoph Muenker, Lukas Wehrstein (added XML ConfigurableConstructor and initial state parameter (on / off))
 * @version 1.0, 07.03.2013
 */
public class OneStateEnergyComponent implements EnergyComponent {

	private DefaultEnergyState state = null;

	private ComponentType componentType = null;

	private EnergyEventListener energyModel;

	boolean on = true;

	/**
	 * Creates the OneStateEnergyComponent with the given power consumption.
	 * Automatically turned on
	 * 
	 * @param energyConsumption
	 *            The basic energy consumption in uWatt.
 *            @param componentType the type of component
	 */
	public OneStateEnergyComponent(double energyConsumptionUwatt,
			ComponentType componentType) {
			this(energyConsumptionUwatt, componentType, true);
	}
	
	/**
	 *  Creates the OneStateEnergyComponent with the given power consunmption and initial state (on/off)
	 * @param energyConsumptionUwatt energy consumption of the component in micro watt
	 * @param componentType type of the component
	 * @param on initial state of the energycomponent
	 */
	public OneStateEnergyComponent(double energyConsumptionUwatt, ComponentType componentType, boolean on) {
		this.componentType = componentType;
		state = new DefaultEnergyState(componentType.toString(),
				energyConsumptionUwatt);
		Event.scheduleWithDelay(Time.SECOND, this, this, 0);
		this.on = on;
	}

	/**
	 * Creates the OneStateEnergyComponent with the given power consumption.
	 * @param energyConsumptionUwatt	The basic energy consumption in uWatt
	 * @param componentTypeName			The componentType name as a string
	 */
	@XMLConfigurableConstructor({ "energyConsumptionUwatt", "componentType"}) // currently not used (LW)
	public OneStateEnergyComponent(double energyConsumptionUwatt, String componentTypeName) {
		this(energyConsumptionUwatt, ComponentType.valueOf(componentTypeName), false);
	}

	
	public DefaultEnergyState getState() {
		return this.state;
	}

	@Override
	public void eventOccurred(Object content, int type) {
		if (isOn()) {
//			double consumedEnergy = state.getEnergyConsumption() * Time.SECOND; 
			// edit LW: multiplication with second means multiplying with 10^6 which leads to wrong energy values...
			double consumedEnergy = state.getEnergyConsumption();
			Monitor.log(getClass(), Level.INFO, "%s consumed %s uJ", componentType, consumedEnergy);
			energyModel.componentConsumedEnergy(this, consumedEnergy);
			
			if (energyModel.componentCanBeActivated(this)) {
				Event.scheduleWithDelay(Time.SECOND, this, null, 0);
			} else {
				this.turnOff();
			}
		}
	}

	@Override
	public ComponentType getType() {
		return componentType;
	}

	@Override
	public boolean turnOff() {
		this.on = false;
		return true;
	}

	@Override
	public boolean turnOn() {
		this.on = true;
		Event.scheduleWithDelay(Time.SECOND, this, this, 0);
		return true;
	}

	@Override
	public boolean isOn() {
		return on;
	}

	@Override
	public void setEnergyEventListener(EnergyEventListener listener) {
		energyModel = listener;
	}

}
