/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.energy.models;

import de.tud.kom.p2psim.api.application.Application;
import de.tud.kom.p2psim.api.common.SimHost;
import de.tud.kom.p2psim.api.energy.Battery;
import de.tud.kom.p2psim.api.energy.EnergyComponent;
import de.tud.kom.p2psim.api.energy.EnergyEventListener;
import de.tud.kom.p2psim.api.network.SimNetInterface;
import de.tud.kom.p2psim.impl.common.ApplicationHost;
import de.tud.kom.p2psim.impl.common.Deactivatable;
import de.tud.kom.p2psim.impl.energy.components.StatefulActuatorComponent;
import de.tud.kom.p2psim.impl.wsn.WSNAppTimer;
import de.tudarmstadt.maki.simonstrator.api.Monitor;
import de.tudarmstadt.maki.simonstrator.api.Monitor.Level;

/**
 * Energy Model based on multiple exchangeable components. Each component states the amount of consumed energy,
 *  which the energy model than will remove from the battery.
 * 
 * @author Julian Zobel, Lukas Wehrstein
 * @version 1.1, 22.03.2022
 */
public class ComponentBasedEnergyModel extends AbstractEnergyModel implements EnergyEventListener {

	public ComponentBasedEnergyModel(SimHost host, Battery bat) {
		super(host, bat);
		
	}

	@Override
	public void registerComponent(EnergyComponent comp) {
		energyComponents.add(comp);
		comp.setEnergyEventListener(this);
		Monitor.log(getClass(), Level.INFO, "Host %s: EnergyComponent %s registered.", host.getId(), comp.getType());
	}

	@Override
	public void componentConsumedEnergy(EnergyComponent component, double consumedEnergy) {		
		if (!bat.isEmpty()) {
			
			bat.consumeEnergy(consumedEnergy);
			Monitor.log(getClass(), Level.INFO, "Host %s: %s consumed %s uJ", host.getId(), component.getType(), consumedEnergy);
			
			monitorEnergyConsumation(component, consumedEnergy);
			String monitorString = "Host " + host.getHostId() + ": " + getCurrentPercentageString("###.##") + "%"; 
			Monitor.log(this.getClass(), Level.DEBUG, "Battery state: %s remaining", monitorString);
			
			if (bat.isEmpty()) {
				
				try {
					if (host instanceof ApplicationHost) {
						Application app = ((ApplicationHost) host).getApplication();
						if (app instanceof WSNAppTimer) {
							((WSNAppTimer) app).stopTimer();
						}else {
							Monitor.log(getClass(), Level.ERROR, "Host %s: application is not instance of WSNAppTimer.", host.getId());
						}
					}else {
						Monitor.log(getClass(), Level.ERROR, "Host %s: host is not instance of ApplicationHost.", host.getId());
					}
				}catch (Exception e) {
					Monitor.log(getClass(), Level.ERROR, "Host %s: Error while deacting host due to discharged battery.", host.getId());
				}
				
				
				monitorEmptyBattery();
				// deactivate host, if possible
				if(host instanceof Deactivatable) {
					((Deactivatable) host).setActive(false);
					Monitor.log(getClass(), Level.WARN, "Host %s has been deactivated due to no energy remaining.", host.getId());
				}else {
					Monitor.log(getClass(), Level.WARN, "Host %s: Host can not be deactivated.", host.getId());
				}
				
				// Movement Speed of inactive Node is set to 0
				host.getTopologyComponent().setMovementSpeed(0);
				
				// turn devices off
				if(component instanceof StatefulActuatorComponent) {
					((StatefulActuatorComponent) component).turnOff();
				}
				
				/*
				 * Battery is now empty. Go offline.
				 */	
				
				for (SimNetInterface net : host.getNetworkComponent()
						.getSimNetworkInterfaces()) {
					net.goOffline();
					
				}
				Monitor.log(ComponentBasedEnergyModel.class, Level.WARN, "Host %s: offline, due to fully discharged battery.", getHost().getHostId());
			}
		}
	}

	@Override
	public boolean componentCanBeActivated(EnergyComponent component) {
		if(!bat.isEmpty()) {
			return true;
		}
		
		return false;
	}
	
}
