/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.wsn;

import de.tud.kom.p2psim.api.energy.ComponentType;
import de.tud.kom.p2psim.api.energy.EnergyComponent;
import de.tud.kom.p2psim.api.energy.EnergyModel;
import de.tud.kom.p2psim.impl.energy.components.OneStateEnergyComponent;
import de.tudarmstadt.maki.simonstrator.api.Host;
import de.tudarmstadt.maki.simonstrator.api.Monitor;
import de.tudarmstadt.maki.simonstrator.api.Monitor.Level;
import de.tudarmstadt.maki.simonstrator.api.component.ComponentNotAvailableException;
import de.tudarmstadt.maki.simonstrator.api.component.HostComponent;
import de.tudarmstadt.maki.simonstrator.api.component.HostComponentFactory;

/**
 * This class represents a generic WSN Host Component
 * @author Lukas Wehrstein
 * @version 1.0, 25.03.2022
 */
public class GenericWSNComponent implements EnergyAwareHostComponent{

	protected EnergyComponent energyComponent;
	protected Host host;
	protected ComponentType componentType;
	protected boolean isActive;
	protected String name;
	
	public GenericWSNComponent(Host host, ComponentType componentType, EnergyComponent energyComponent) {
		this.energyComponent = energyComponent;
		this.host = host;
		this.componentType = componentType;
		this.name = componentType.name();
		this.isActive = false;
		energyComponent.turnOff();
	}
	
	@Override
	public void initialize() {
		
		if(!isActive) {
			// component is turned off
			
		}
		try {
			EnergyModel energyModel = host.getComponent(EnergyModel.class);
			energyModel.registerComponent(this.energyComponent);
			
			if(energyComponent instanceof OneStateEnergyComponent) {
				OneStateEnergyComponent osec = (OneStateEnergyComponent) energyComponent;
				Monitor.log(getClass(), Level.DEBUG, "Component %s was registered with %s uJ", name, osec.getState().getEnergyConsumption());
			}
			
		} catch (ComponentNotAvailableException e) {
			Monitor.log(getClass(), Level.WARN, "Host %s: This host does not have a valid energy model", host.getId());
		}

	}

	@Override
	public void shutdown() {
		setActive(false);

	}

	@Override
	public Host getHost() {
		return this.host;
	}

	@Override
	public boolean isActive() {
		return this.isActive;
	}

	@Override
	public void setActive(boolean b) {
		if(isActive != b) {
			// change state
			this.isActive = b;
			if(b) {
				getEnergyComponent().turnOn();
			}else {
				getEnergyComponent().turnOff();
			}
		}
	}

	@Override
	public EnergyComponent getEnergyComponent() {
		return this.energyComponent;
	}

	/**
	 * EnergyComponent that represents this specific sensor component
	 * @param energyComponent
	 */
	@Override
	public void setEnergyComponent(EnergyComponent energyComponent) {
		this.energyComponent = energyComponent;
	}
	
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}
	
	
	public static class Factory implements HostComponentFactory{
		EnergyComponent energyComponent;
		ComponentType componentType;
		double energyConsumptionUwatt;
		
		/**
		 * Sets the component type of the genericWSNComponent. BASIC if parameter is not known
		 * @param componentType
		 */
		public void setComponentType(String componentType) {
			try {
				this.componentType = ComponentType.valueOf(componentType);
			}catch (Exception e) {
				Monitor.log(getClass(), Level.ERROR, "%s is not a valid ComponentType. Using BASIC instead", componentType);
				this.componentType = ComponentType.BASIC;
			}
		}
		
		/**
		 * Defines the energy consumption of this sensor
		 * @param energyConsumptionUwatt
		 */
		public void setEnergyConsumptionUwatt(double energyConsumptionUwatt) {
			Monitor.log(getClass(), Level.DEBUG, "EnergyConsumption of this %s component was set to %s uW", componentType, this.energyConsumptionUwatt);
			this.energyConsumptionUwatt = energyConsumptionUwatt;
		}
		
		@Override
		public HostComponent createComponent(Host host) {
			this.energyComponent = new OneStateEnergyComponent(energyConsumptionUwatt, componentType, false);
			return new GenericWSNComponent(host, componentType, energyComponent);
		}
		
	}

}
