/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.wsn.microcontroller;

import java.util.LinkedList;
import java.util.List;

import de.tud.kom.p2psim.api.energy.ComponentType;
import de.tud.kom.p2psim.api.energy.EnergyComponent;
import de.tud.kom.p2psim.api.energy.EnergyModel;
import de.tud.kom.p2psim.api.energy.EnergyState;
import de.tud.kom.p2psim.impl.energy.DefaultEnergyState;
import de.tud.kom.p2psim.impl.energy.components.MultiStateEnergyComponent;
import de.tud.kom.p2psim.impl.energy.components.OneStateEnergyComponent;
import de.tud.kom.p2psim.impl.wsn.EnergyAwareHostComponent;
import de.tudarmstadt.maki.simonstrator.api.Host;
import de.tudarmstadt.maki.simonstrator.api.Monitor;
import de.tudarmstadt.maki.simonstrator.api.Monitor.Level;
import de.tudarmstadt.maki.simonstrator.api.component.ComponentNotAvailableException;
import de.tudarmstadt.maki.simonstrator.api.component.HostComponent;
import de.tudarmstadt.maki.simonstrator.api.component.HostComponentFactory;

/**
 * This class represents a generic microcontroller model
 * @author Lukas Wehrstein
 * @version 1.0, 22.03.2022
 */
public class GenericMicrocontroller implements EnergyAwareHostComponent {

	private Host host;
	private EnergyComponent energyComponent;
	
	private boolean active;

	/**
	 * Creates a Generic Microcontroller with one energy State
	 * @param host
	 * @param energyConsumptionActive in uW
	 */
	public GenericMicrocontroller(Host host, double energyConsumptionActive) {
		this.host = host;
		this.active = true;
		energyComponent = new OneStateEnergyComponent(energyConsumptionActive, ComponentType.MICROCONTROLLER);
	}
	
	/**
	 * Creates a generic Microcontroller with two states - active and sleep
	 * @param host
	 * @param voltageActive				in mV
	 * @param currentConsumptionActive	in mA
	 * @param voltageSleep				in mV
	 * @param currentConsumptionSleep	in mA
	 */
	public GenericMicrocontroller(Host host, double voltageActive, double currentConsumptionActive, double voltageSleep, double currentConsumptionSleep) {
		this(host, voltageActive * currentConsumptionActive, voltageSleep * currentConsumptionSleep);
	}
	
	/**
	 * Creates a generic Microcontroller with two states - active and sleep 
	 * @param host
	 * @param energyConsumptionActive in uW
	 * @param energyConsumptionSleep  in uW
	 */
	public GenericMicrocontroller(Host host, double energyConsumptionActive, double energyConsumptionSleep) {
		this.host = host;
		this.active = true;
		
		EnergyState stateActive = new DefaultEnergyState("active", energyConsumptionActive);
		EnergyState stateSleep = new DefaultEnergyState("sleep", energyConsumptionSleep);
		
		List<EnergyState> states = new LinkedList<>();
		states.add(stateActive);
		states.add(stateSleep);
		
		this.energyComponent = new MultiStateEnergyComponent(states, ComponentType.MICROCONTROLLER);
	}

	@Override
	public void initialize() {
		
		try {
			EnergyModel energyModel = host.getComponent(EnergyModel.class);
			energyModel.registerComponent(this.energyComponent);
			
		} catch (ComponentNotAvailableException e) {
			// TODO: handle exception
		}
		
		Monitor.log(getClass(), Level.DEBUG, "Host %s: GenericMicrocontroller has been initialized.", this.host.getId());
		
	}

	@Override
	public void shutdown() {
		setActive(false);
		Monitor.log(getClass(), Level.DEBUG, "Host %s: GenericMicrocontroller has been shut down.", this.host.getId());
	}

	@Override
	public Host getHost() {
		return host;
	}

	@Override
	public boolean isActive() {
		return active;
	}

	@Override
	public void setActive(boolean active) {
		this.active = active;
		
	}

	@Override
	public EnergyComponent getEnergyComponent() {
		return this.energyComponent;
	}

	@Override
	public void setEnergyComponent(EnergyComponent energyComponent) {
		this.energyComponent = energyComponent;
		
	}
	
	public static class Factory implements HostComponentFactory{
		
		double energyConsumptionUwattActive;
		double energyConsumptionUwattSleep;
		
		/**
		 * Sets the energyConsumption for active state
		 * @param energyConsumptionUwattActive in uW
		 */
		public void setEnergyConsumptionActiveUwatt(double energyConsumptionUwattActive){
			this.energyConsumptionUwattActive = energyConsumptionUwattActive;
		}
		
		/**
		 * Sets the energy Consumption for sleep state
		 * @param energyConsumptionUwattSleep in uW
		 */
		public void setEnergyConsumptionSleepUwatt(double energyConsumptionUwattSleep){
			this.energyConsumptionUwattSleep = energyConsumptionUwattSleep;
		}
		
		@Override
		public HostComponent createComponent(Host host) {
			return new GenericMicrocontroller(host, energyConsumptionUwattActive, energyConsumptionUwattSleep);
		}
		
	}


}
